@@ -149,7 +149,8 @@ class  PreEscape : public TemplateModifier { MODIFY_SIGNATURE_; };
 extern  PreEscape pre_escape;
 
 // Like HtmlEscape but allows HTML entities, <br> tags, <wbr> tags,
-// matched <b> and </b> tags, and matched <i> and </i> tags.
+// matched <b> and </b> tags, matched <i> and </i> tags, matched <em> and </em>
+// tags, and matched <span dir=(rtl|ltr)> tags.
 class  SnippetEscape : public TemplateModifier { MODIFY_SIGNATURE_; };
 extern  SnippetEscape snippet_escape;
 
@@ -182,18 +183,31 @@ extern  CleanseCss cleanse_css;
 // url that doesn't have a protocol hidden in it (ie [foo.html] is
 // fine, but not [javascript:foo]) and then performs another type of
 // escaping. Returns the url escaped with the specified modifier if
-// good, otherwise returns "#".
+// good, otherwise returns a safe replacement URL.
+// This is normally "#", but for <img> tags, it is not safe to set
+// the src attribute to "#".  This is because this causes some browsers
+// to reload the page, which can cause a DoS.
 class  ValidateUrl : public TemplateModifier {
  public:
-  explicit ValidateUrl(const TemplateModifier& chained_modifier)
-      : chained_modifier_(chained_modifier) { }
+  explicit ValidateUrl(const TemplateModifier& chained_modifier,
+                       const char* unsafe_url_replacement)
+      : chained_modifier_(chained_modifier),
+        unsafe_url_replacement_(unsafe_url_replacement),
+        unsafe_url_replacement_length_(strlen(unsafe_url_replacement)) { }
   MODIFY_SIGNATURE_;
+  static const char* const kUnsafeUrlReplacement;
+  static const char* const kUnsafeImgSrcUrlReplacement;
  private:
   const TemplateModifier& chained_modifier_;
+  const char* unsafe_url_replacement_;
+  int unsafe_url_replacement_length_;
 };
 extern  ValidateUrl validate_url_and_html_escape;
 extern  ValidateUrl validate_url_and_javascript_escape;
 extern  ValidateUrl validate_url_and_css_escape;
+extern  ValidateUrl validate_img_src_url_and_html_escape;
+extern  ValidateUrl validate_img_src_url_and_javascript_escape;
+extern  ValidateUrl validate_img_src_url_and_css_escape;
 
 // Escapes < > & " ' to &lt; &gt; &amp; &quot; &#39; (same as in HtmlEscape).
 // If you use it within a CDATA section, you may be escaping more characters
