@@ -133,6 +133,53 @@ class TemplateModifiersUnittest {
     dict.SetEscapedValue("close i b",
                          "<b><i>foo",
                          snippet_escape);
+    dict.SetEscapedValue("em",
+                         "<em>foo</em>",
+                         snippet_escape);
+    dict.SetEscapedValue("nested em",
+                         "<b>This is foo<em>...</em></b>",
+                         snippet_escape);
+    dict.SetEscapedValue("unclosed em",
+                         "<em>foo",
+                         snippet_escape);
+    dict.SetEscapedValue("wrongly closed em",
+                         "foo</em>",
+                         snippet_escape);
+    dict.SetEscapedValue("misnested em",
+                         "<i><em>foo</i></em>",
+                         snippet_escape);
+    dict.SetEscapedValue("span ltr",
+                         "<span dir=ltr>bidi text</span>",
+                         snippet_escape);
+    dict.SetEscapedValue("span rtl",
+                         "<span dir=rtl>bidi text</span>",
+                         snippet_escape);
+    dict.SetEscapedValue("span garbage dir attr",
+                         "<span dir=foo>bidi text</span>",
+                         snippet_escape);
+    dict.SetEscapedValue("span no dir",
+                         "<span>bidi text</span>",
+                         snippet_escape);
+    dict.SetEscapedValue("span bad attribute",
+                         "<span onclick=alert('foo')>bidi text</span>",
+                         snippet_escape);
+    dict.SetEscapedValue("span quotes",
+                         "<span dir=\"rtl\">bidi text</span>",
+                         snippet_escape);
+    dict.SetEscapedValue("nested span",
+                         "<b>This is <span dir=rtl>bidi text</span></b>",
+                         snippet_escape);
+    dict.SetEscapedValue("doubly-nested span",
+                         "<span dir=rtl>This is <span dir=rtl>"
+                         "bidi text</span></span>",
+                         snippet_escape);
+    dict.SetEscapedValue("two spans",
+                         "<b>This is <span dir=rtl>text</span> that is "
+                         "<span dir=rtl>bidi.</span></b>",
+                         snippet_escape);
+    dict.SetEscapedValue("unclosed span",
+                         "<b>This is <span dir=rtl>bidi text",
+                         snippet_escape);
 
     TemplateDictionaryPeer peer(&dict);  // peer can look inside the dict
     ASSERT_STREQ(peer.GetSectionValue("easy snippet"), "foo");
@@ -158,6 +205,34 @@ class TemplateModifiersUnittest {
     ASSERT_STREQ(peer.GetSectionValue("unterminated 5"), "<b>foo&lt;/b</b>");
     ASSERT_STREQ(peer.GetSectionValue("close b i"), "<i><b>foo</b></i>");
     ASSERT_STREQ(peer.GetSectionValue("close i b"), "<b><i>foo</i></b>");
+    ASSERT_STREQ(peer.GetSectionValue("em"), "<em>foo</em>");
+    ASSERT_STREQ(peer.GetSectionValue("nested em"),
+                 "<b>This is foo<em>...</em></b>");
+    ASSERT_STREQ(peer.GetSectionValue("unclosed em"), "<em>foo</em>");
+    ASSERT_STREQ(peer.GetSectionValue("wrongly closed em"), "foo&lt;/em&gt;");
+    ASSERT_STREQ(peer.GetSectionValue("misnested em"), "<i><em>foo</i></em>");
+    ASSERT_STREQ(peer.GetSectionValue("span ltr"),
+                 "<span dir=ltr>bidi text</span>");
+    ASSERT_STREQ(peer.GetSectionValue("span rtl"),
+                 "<span dir=rtl>bidi text</span>");
+    ASSERT_STREQ(peer.GetSectionValue("span garbage dir attr"),
+                 "&lt;span dir=foo&gt;bidi text&lt;/span&gt;");
+    ASSERT_STREQ(peer.GetSectionValue("span no dir"),
+                 "&lt;span&gt;bidi text&lt;/span&gt;");
+    ASSERT_STREQ(peer.GetSectionValue("span bad attribute"),
+                 "&lt;span onclick=alert(&#39;foo&#39;)&gt;bidi text&lt;/span&gt;");
+    ASSERT_STREQ(peer.GetSectionValue("span quotes"),
+                 "&lt;span dir=&quot;rtl&quot;&gt;bidi text&lt;/span&gt;");
+    ASSERT_STREQ(peer.GetSectionValue("nested span"),
+                 "<b>This is <span dir=rtl>bidi text</span></b>");
+    ASSERT_STREQ(peer.GetSectionValue("doubly-nested span"),
+                 "<span dir=rtl>This is &lt;span dir=rtl&gt;bidi text"
+                 "</span>&lt;/span&gt;");
+    ASSERT_STREQ(peer.GetSectionValue("two spans"),
+                 "<b>This is <span dir=rtl>text</span> that is "
+                 "<span dir=rtl>bidi.</span></b>");
+    ASSERT_STREQ(peer.GetSectionValue("unclosed span"),
+                 "<b>This is <span dir=rtl>bidi text</span></b>");
   }
 
   static void TestPreEscape() {
@@ -230,9 +305,49 @@ class TemplateModifiersUnittest {
     ASSERT_STREQ(peer.GetSectionValue("harder https URL"),
                  "https://www.google.com/search?q=f&amp;hl=en");
     ASSERT_STREQ(peer.GetSectionValue("easy javascript URL"),
-                 "#");
+                 ctemplate::ValidateUrl::kUnsafeUrlReplacement);
     ASSERT_STREQ(peer.GetSectionValue("harder javascript URL"),
-                 "#");
+                 ctemplate::ValidateUrl::kUnsafeUrlReplacement);
+    ASSERT_STREQ(peer.GetSectionValue("easy relative URL"),
+                 "foobar.html");
+    ASSERT_STREQ(peer.GetSectionValue("harder relative URL"),
+                 "/search?q=green flowers&amp;hl=en");
+    ASSERT_STREQ(peer.GetSectionValue("ftp URL"),
+                 "ftp://ftp.example.org/pub/file.txt");
+  }
+
+  static void TestValidateImgSrcUrlHtmlEscape() {
+    TemplateDictionary dict("TestValidateImgSrcUrlHtmlEscape", NULL);
+    dict.SetEscapedValue("easy http URL", "http://www.google.com",
+                         ctemplate::validate_img_src_url_and_html_escape);
+    dict.SetEscapedValue("harder https URL",
+                         "https://www.google.com/search?q=f&hl=en",
+                         ctemplate::validate_img_src_url_and_html_escape);
+    dict.SetEscapedValue("easy javascript URL",
+                         "javascript:alert(document.cookie)",
+                         ctemplate::validate_img_src_url_and_html_escape);
+    dict.SetEscapedValue("harder javascript URL",
+                         "javascript:alert(10/5)",
+                         ctemplate::validate_img_src_url_and_html_escape);
+    dict.SetEscapedValue("easy relative URL",
+                         "foobar.html",
+                         ctemplate::validate_img_src_url_and_html_escape);
+    dict.SetEscapedValue("harder relative URL",
+                         "/search?q=green flowers&hl=en",
+                         ctemplate::validate_img_src_url_and_html_escape);
+    dict.SetEscapedValue("ftp URL",
+                         "ftp://ftp.example.org/pub/file.txt",
+                         ctemplate::validate_img_src_url_and_html_escape);
+
+    TemplateDictionaryPeer peer(&dict);  // peer can look inside the dict
+    ASSERT_STREQ(peer.GetSectionValue("easy http URL"),
+                 "http://www.google.com");
+    ASSERT_STREQ(peer.GetSectionValue("harder https URL"),
+                 "https://www.google.com/search?q=f&amp;hl=en");
+    ASSERT_STREQ(peer.GetSectionValue("easy javascript URL"),
+                 ctemplate::ValidateUrl::kUnsafeImgSrcUrlReplacement);
+    ASSERT_STREQ(peer.GetSectionValue("harder javascript URL"),
+                 ctemplate::ValidateUrl::kUnsafeImgSrcUrlReplacement);
     ASSERT_STREQ(peer.GetSectionValue("easy relative URL"),
                  "foobar.html");
     ASSERT_STREQ(peer.GetSectionValue("harder relative URL"),
@@ -290,20 +405,84 @@ class TemplateModifiersUnittest {
                  "https://www.google.com/search?q\\x3df\\x26hl\\x3den");
     ASSERT_STREQ(peer.GetSectionValue("mangled http URL"),
                  "HTTP://www.google.com");
-    ASSERT_STREQ(peer.GetSectionValue("easy javascript URL"),
-                 "#");
+    ASSERT_STREQ(peer.GetSectionValue("easy javascript URL"), "#");
     ASSERT_STREQ(peer.GetSectionValue("harder javascript URL"),
-                 "#");
+                 ctemplate::ValidateUrl::kUnsafeUrlReplacement);
     ASSERT_STREQ(peer.GetSectionValue("easy relative URL"),
                  "foobar.html");
     ASSERT_STREQ(peer.GetSectionValue("harder relative URL"),
                  "/search?q\\x3dgreen flowers\\x26hl\\x3den");
     ASSERT_STREQ(peer.GetSectionValue("data URL"),
-                 "#");
+                 ctemplate::ValidateUrl::kUnsafeUrlReplacement);
     ASSERT_STREQ(peer.GetSectionValue("mangled javascript URL"),
-                 "#");
+                 ctemplate::ValidateUrl::kUnsafeUrlReplacement);
     ASSERT_STREQ(peer.GetSectionValue("harder mangled javascript URL"),
-                 "#");
+                 ctemplate::ValidateUrl::kUnsafeUrlReplacement);
+  }
+
+  static void TestValidateImgSrcUrlJavascriptEscape() {
+    TemplateDictionary dict("TestValidateImgSrcUrlJavascriptEscape", NULL);
+    dict.SetEscapedValue(
+        "easy http URL", "http://www.google.com",
+        ctemplate::validate_img_src_url_and_javascript_escape);
+    dict.SetEscapedValue(
+        "harder https URL",
+        "https://www.google.com/search?q=f&hl=en",
+        ctemplate::validate_img_src_url_and_javascript_escape);
+    dict.SetEscapedValue(
+        "mangled http URL", "HTTP://www.google.com",
+        ctemplate::validate_img_src_url_and_javascript_escape);
+    dict.SetEscapedValue(
+        "easy javascript URL",
+        "javascript:alert(document.cookie)",
+        ctemplate::validate_img_src_url_and_javascript_escape);
+    dict.SetEscapedValue(
+        "harder javascript URL",
+        "javascript:alert(10/5)",
+        ctemplate::validate_img_src_url_and_javascript_escape);
+    dict.SetEscapedValue(
+        "easy relative URL",
+        "foobar.html",
+        ctemplate::validate_img_src_url_and_javascript_escape);
+    dict.SetEscapedValue(
+        "harder relative URL",
+        "/search?q=green flowers&hl=en",
+        ctemplate::validate_img_src_url_and_javascript_escape);
+    dict.SetEscapedValue(
+        "data URL",
+        "data: text/html",
+        ctemplate::validate_img_src_url_and_javascript_escape);
+    dict.SetEscapedValue(
+        "mangled javascript URL",
+        "javaSCRIPT:alert(5)",
+        ctemplate::validate_img_src_url_and_javascript_escape);
+    dict.SetEscapedValue(
+        "harder mangled javascript URL",
+        "java\nSCRIPT:alert(5)",
+        ctemplate::validate_img_src_url_and_javascript_escape);
+
+
+    TemplateDictionaryPeer peer(&dict);  // peer can look inside the dict
+    ASSERT_STREQ(peer.GetSectionValue("easy http URL"),
+                 "http://www.google.com");
+    ASSERT_STREQ(peer.GetSectionValue("harder https URL"),
+                 "https://www.google.com/search?q\\x3df\\x26hl\\x3den");
+    ASSERT_STREQ(peer.GetSectionValue("mangled http URL"),
+                 "HTTP://www.google.com");
+    ASSERT_STREQ(peer.GetSectionValue("easy javascript URL"),
+                 ctemplate::ValidateUrl::kUnsafeImgSrcUrlReplacement);
+    ASSERT_STREQ(peer.GetSectionValue("harder javascript URL"),
+                 ctemplate::ValidateUrl::kUnsafeImgSrcUrlReplacement);
+    ASSERT_STREQ(peer.GetSectionValue("easy relative URL"),
+                 "foobar.html");
+    ASSERT_STREQ(peer.GetSectionValue("harder relative URL"),
+                 "/search?q\\x3dgreen flowers\\x26hl\\x3den");
+    ASSERT_STREQ(peer.GetSectionValue("data URL"),
+                 "/images/cleardot.gif");
+    ASSERT_STREQ(peer.GetSectionValue("mangled javascript URL"),
+                 ctemplate::ValidateUrl::kUnsafeImgSrcUrlReplacement);
+    ASSERT_STREQ(peer.GetSectionValue("harder mangled javascript URL"),
+                 ctemplate::ValidateUrl::kUnsafeImgSrcUrlReplacement);
   }
 
   static void TestValidateUrlCssEscape() {
@@ -327,7 +506,38 @@ class TemplateModifiersUnittest {
                  "http://www.google.com");
     ASSERT_STREQ(peer.GetSectionValue("harder https URL"),
                  "https://www.google.com/search?q=f&hl=en");
-    ASSERT_STREQ(peer.GetSectionValue("javascript URL"), "#");
+    ASSERT_STREQ(peer.GetSectionValue("javascript URL"),
+                 ctemplate::ValidateUrl::kUnsafeUrlReplacement);
+    ASSERT_STREQ(peer.GetSectionValue("relative URL"),
+                 "/search?q=green flowers&hl=en");
+    ASSERT_STREQ(peer.GetSectionValue("hardest URL"),
+                 "http://www.google.com/s?q=%27bla%27"
+                 "&a=%22%22&b=%28%3Ctag%3E%29&c=%2A%0D%0A%5C%5Cbla");
+  }
+
+  static void TestValidateImgSrcUrlCssEscape() {
+    TemplateDictionary dict("TestValidateImgSrcUrlCssEscape", NULL);
+    dict.SetEscapedValue("easy http URL", "http://www.google.com",
+                         ctemplate::validate_img_src_url_and_css_escape);
+    dict.SetEscapedValue("harder https URL",
+                         "https://www.google.com/search?q=f&hl=en",
+                         ctemplate::validate_img_src_url_and_css_escape);
+    dict.SetEscapedValue("javascript URL",
+                         "javascript:alert(document.cookie)",
+                         ctemplate::validate_img_src_url_and_css_escape);
+    dict.SetEscapedValue("relative URL", "/search?q=green flowers&hl=en",
+                         ctemplate::validate_img_src_url_and_css_escape);
+    dict.SetEscapedValue("hardest URL", "http://www.google.com/s?q='bla'"
+                         "&a=\"\"&b=(<tag>)&c=*\r\n\\\\bla",
+                         ctemplate::validate_img_src_url_and_css_escape);
+
+    TemplateDictionaryPeer peer(&dict);  // peer can look inside the dict
+    ASSERT_STREQ(peer.GetSectionValue("easy http URL"),
+                 "http://www.google.com");
+    ASSERT_STREQ(peer.GetSectionValue("harder https URL"),
+                 "https://www.google.com/search?q=f&hl=en");
+    ASSERT_STREQ(peer.GetSectionValue("javascript URL"),
+                 ctemplate::ValidateUrl::kUnsafeImgSrcUrlReplacement);
     ASSERT_STREQ(peer.GetSectionValue("relative URL"),
                  "/search?q=green flowers&hl=en");
     ASSERT_STREQ(peer.GetSectionValue("hardest URL"),
@@ -782,8 +992,8 @@ class TemplateModifiersUnittest {
     ASSERT(CheckXSSAlternative("url_query_escape", "",
                                "url_escape_with_arg", "=query"));
 
-    // H=(pre|snippet|attribute), p, u, U=query and U=html (a.k.a H=url)
-    // are all alternatives to h.
+    // H=(pre|snippet|attribute), p, u, U=query, U=html (a.k.a H=url)
+    // and I=html are all alternatives to h.
     ASSERT(CheckXSSAlternative("h", "", "H", "=pre"));
     ASSERT(CheckXSSAlternative("h", "", "H", "=snippet"));
     ASSERT(CheckXSSAlternative("h", "", "H", "=attribute"));
@@ -792,10 +1002,12 @@ class TemplateModifiersUnittest {
     ASSERT(CheckXSSAlternative("h", "", "u", ""));
     ASSERT(CheckXSSAlternative("h", "", "U", "=query"));
     ASSERT(CheckXSSAlternative("h", "", "U", "=html"));
+    ASSERT(CheckXSSAlternative("h", "", "I", "=html"));
 
-    // But h is not an alternative to H=attribute
+    // But h is not an alternative to H=attribute and I=html,
     // nor is json_escape an alternative to h.
     ASSERT(!CheckXSSAlternative("H", "=attribute", "h", ""));
+    ASSERT(!CheckXSSAlternative("I", "=html", "h", ""));
     ASSERT(!CheckXSSAlternative("h", "", "json_escape", ""));
 
     // H=snippet and H=attribute are alternatives to H=pre
@@ -808,6 +1020,19 @@ class TemplateModifiersUnittest {
     ASSERT(CheckXSSAlternative("json_escape", "", "javascript_escape", ""));
     ASSERT(CheckXSSAlternative("javascript_escape", "", "json_escape", ""));
 
+    // I=javascript is an alternative to :j and :U=javascript but not
+    // vice versa
+    ASSERT(CheckXSSAlternative("javascript_escape", "", "I", "=javascript"));
+    ASSERT(CheckXSSAlternative("U", "=javascript", "I", "=javascript"));
+    ASSERT(!CheckXSSAlternative("I", "=javascript", "javascript_escape", ""));
+    ASSERT(!CheckXSSAlternative("I", "=javascript", "U", "=javascript"));
+
+    // U=css and I=css are alternatives to :c but not vice versa
+    ASSERT(CheckXSSAlternative("c", "", "U", "=css"));
+    ASSERT(CheckXSSAlternative("c", "", "I", "=css"));
+    ASSERT(!CheckXSSAlternative("U", "=css", "c", ""));
+    ASSERT(!CheckXSSAlternative("I", "=css", "c", ""));
+
     // Extended modifier should not match any other except itself.
     ASSERT(!CheckXSSAlternative("x-bla", "", "x-foo", ""));
   }
@@ -900,8 +1125,11 @@ int main(int argc, char** argv) {
   TemplateModifiersUnittest::TestPreEscape();
   TemplateModifiersUnittest::TestXmlEscape();
   TemplateModifiersUnittest::TestValidateUrlHtmlEscape();
+  TemplateModifiersUnittest::TestValidateImgSrcUrlHtmlEscape();
   TemplateModifiersUnittest::TestValidateUrlJavascriptEscape();
+  TemplateModifiersUnittest::TestValidateImgSrcUrlJavascriptEscape();
   TemplateModifiersUnittest::TestValidateUrlCssEscape();
+  TemplateModifiersUnittest::TestValidateImgSrcUrlCssEscape();
   TemplateModifiersUnittest::TestCleanseAttribute();
   TemplateModifiersUnittest::TestCleanseCss();
   TemplateModifiersUnittest::TestJavascriptEscape();
